'''
PyTorn v0.0.1 (under development) by Joshua Li
Torn City game data extraction and interpretation made simple.

TODO:
maybe just make calls to all torn data fields in the beginning to load all the torn data? and then make a reload function?
make ID mappings for properties, stocks, etc. everything in torn data fields AND the complimentary mapping methods
provide methods to list selection possibilities for each call
incorporate automatic rate limiting
'''
import requests
from exceptions import TornException


class TornClient(object):
    _BASE_API_URL = "https://api.torn.com"

    def __init__(self, api_key):
        self._key = api_key
        self._item_map = self._build_item_map()

    def get_raw_user_data(self, user_id, field):
        return self._make_request(user_id, 'user', field)

    def get_raw_property_data(self, property_id, field):
        return self._make_request(property_id, 'property', field)

    def get_raw_faction_data(self, faction_id, field):
        return self._make_request(faction_id, 'faction', field)

    def get_raw_company_data(self, company_id, field):
        return self._make_request(company_id, 'company', field)

    def get_raw_market_data(self, item_id, field):
        return self._make_request(item_id, 'market', field)

    def get_raw_torn_data(self, field):
        return self._make_request('', 'torn', field)

    def get_item_id(self, item_name):
        '''
        Returns a numeric string denoting the item id associated with
        an item name.
        '''
        try:
            int(item_name)
        except ValueError:
            return self._item_map[self._clean_item_name(item_name)]
        return str(item_name)

    # more id mappers go here

    def get_item_bazaar_prices(self, item, limit=5):
        '''
        Returns the lowest bazaar prices of an item associated with a valid
        item ID or name.
        '''
        iid = self.get_item_id(item)
        listings = self.get_raw_market_data(iid, 'bazaar')['bazaar']
        return [i[1] for i in sorted(listings.items(),
                key=lambda l: l[1]['cost'])][:limit]

    def _make_request(self, id, mode, field=''):
        # print("*** API CALLED ***")
        url = "{}/{}/{}?selections={}&key={}".format(self._BASE_API_URL, mode,
                                                     id, field, self._key)
        j = requests.get(url).json()
        if 'error' in j:
            raise TornException(j['error'])
        return j

    def _build_item_map(self):
        '''
        Builds a lookup table that maps torn item names to item ids, because
        the Torn API only accepts numeric ids as identification.
        '''
        item_map, item_data = dict(), self.get_raw_torn_data('items')['items']
        for iid in item_data:
            item_map[self._clean_item_name(item_data[iid]['name'])] = iid
        return item_map

    # more map builders go here

    def _clean_item_name(self, item):
        # TODO make this a universal name cleaner since will be adding lots of name to id maps soon?
        for char in "!?',:/ ":  # Can't exclude + or - due to blood bags.
            item = item.replace(char, '')
        return item.upper()

    # more name cleaners go here, or make a universal one
